/*
Melody 1.0
 17.04.2014 Reinhard Atzbach
*/

const int speakerPin = 2;    // Lautsprecheranschluss
const int kammerTon  = 440;  // Hz
const int beats      = 120;  // Tempo (Schläge pro Minute)
const int staccato   = 5;    // Trennpause in Prozent der Tonlänge (nur bei ton())

/*
Codierung der Töne im toneCode-Feld: 
Notenname, Oktave, Vorzeichen oder leer, Notenwert, Punktierung
"C3#v." bedeutet cis der dritten Oktave als punktierte Viertel    
"D2 g" bedeutet D der zweiten Oktave als ganze Note (Leerzeichen erforderlich)
Melodien können etwa 100 Noten umfassen, werden es mehr, bleibt der Arduino stumm.
*/

String toneCode[] = {        
  "F4 a.","D4 s ",           
  "B3bv" ,"D4 v" ,"F4 v" ,   
  "B4bh" ,"D5 a.","C5 s ",  
  "B4bv" ,"D4 v" ,"E4 v" ,
  "F4 h" ,"F4 a" ,"F4 a" ,
  "D5 v.","C5 a" ,"B4bv" ,
  "A4 h" ,"G4 a" ,"A4 a" ,
  "B4bv" ,"B4bv" ,"F4 v" ,
  "D4 v" ,"B3bv" ,
  "W",                        // Toncode 'W' sagt: Wiederholung ab Anfang

  "D5 a" ,"D5 a" ,
  "D5 v" ,"E5bv" ,"F5 v" ,
  "F5 h" ,"E5ba" ,"D5 a" ,
  "C5 v" ,"D5 v" ,"E5bv" ,
  "E5bh" ,"E5bv" ,
  "D5 v.","C5 a" ,"B4bv" ,
  "A4 h" ,"G4 a" ,"A4 a" ,
  "B4bv" ,"D4 v" ,"E4 v" ,
  "F4 h" ,"F4 v" ,
  "B4bv" ,"B4bv" ,"B4ba" ,"A4 a" ,
  "G4 v" ,"G4 v" ,"G4 v" ,
  "C5 v" ,"E5ba" ,"D5 a" ,"C5 a" ,"B4ba" ,
  "B4bv" ,"A4 v" ,"F4 a" ,"F4 a" ,
  "B4bv.","C5 a" ,"D5 a" ,"E5ba" ,
  "F5 h" ,"B4ba" ,"C5 a" ,
  "D5 v.","E5ba" ,"C5 v" ,
  "B4bh.", 
  "0"                                  // Toncode 0 sagt: Hier ist die Melodie zu Ende 
};                                     // (Abbruch der For-Schleife in der Loop)

unsigned int frequenz(String Code) {
  int toneNr;
  if (Code.charAt(0)=='C') toneNr=0;    // charAt(0) = Erstes Zeichen im Notencode
  if (Code.charAt(0)=='D') toneNr=2;    // Halbtöne sind durchnummeriert
  if (Code.charAt(0)=='E') toneNr=4;    // C0 hat Nummer 0, C1 hat 12, C2 hat 24 ...
  if (Code.charAt(0)=='F') toneNr=5;    
  if (Code.charAt(0)=='G') toneNr=7;
  if (Code.charAt(0)=='A') toneNr=9;
  if (Code.charAt(0)=='B') toneNr=11;
  if (Code.charAt(0)=='H') toneNr=11; 
  if (Code.charAt(0)=='P') toneNr=999;  // das P steht für Pause
  int Oktave = int(Code.charAt(1)-47);  // charAt(1) = 2. Zeichen im Notencode
  toneNr=toneNr+12*Oktave;              // fortlaufende Tonnummer von C0 aus
  if (Code.charAt(2)=='#') toneNr++;    // charAt(2) = 3. Zeichen im Notencode
  if (Code.charAt(2)=='b') toneNr--;
  double freq;            // Frequenz berechnen. Kammerton a' mit 440 Hz ist Ton Nr. 69 
  if (toneNr==999) {      // das a eine Oktave höher hat die Frequenz 440 * 2
    freq=0;               // das Bb einen Halbton höher hat die Frequenz 440 * 2 ^(1/12)
  } else {                // die Frequenz eines beliebigen Tons ist 440 * 2 ^ ((Tonnummer-69)/12)              
    freq=kammerTon*pow(2.0,((toneNr-69.0)/12.0));  
  }
  return int(freq); 
}

unsigned long dauer(String Code) {
  long toneLen;                                       // Länge des aktuellen Tons
  long beatLen = 60000/beats;                         // 1 Minute = 60000 Millisekunden
  if (Code.charAt(3)=='g') toneLen=4*beatLen;         // charAt(3) = Vertes Zeichen im Notencode
  if (Code.charAt(3)=='h') toneLen=2*beatLen;
  if (Code.charAt(3)=='v') toneLen=1*beatLen;
  if (Code.charAt(3)=='a') toneLen=beatLen/2;
  if (Code.charAt(3)=='s') toneLen=beatLen/4;
  if (Code.charAt(4)=='.') toneLen=toneLen+toneLen/2; // Punktierte Note = anderthalbfache Länge
  return toneLen;
}

void ton(long Pin, long freq, unsigned long laenge) {
  if (freq==0) {                          // das ist eine Pause
    delay(laenge);
  } 
  else {                             
    long beatMicros = 1000000/freq;       // Dauer eines Beats: 1 Million Mikrosekunden durch Frequenz 
    unsigned long beginn;
    beginn = millis();                    // Aktuelle Einschaltzeit in Millisekunden
    while (millis()<beginn+laenge*(100-staccato)/100) {
      digitalWrite(Pin,HIGH);             // Erste Hälfte des Beats Lautsprecher HIGH
      delayMicroseconds(beatMicros/2);
      digitalWrite(Pin,LOW);              // Zweite Hälfte des Beats Lautsprecher LOW
      delayMicroseconds(beatMicros/2);
    }
    delay(staccato*laenge/100);
  } 
}  

void setup() {
  pinMode(speakerPin,OUTPUT);
}

void loop() {
  boolean wiederholung=false;
  for(int n=0;toneCode[n].charAt(0)!='0';n++) {     // Schleife läuft bis Code ' 0'erkannt wird
    if (toneCode[n].charAt(0)=='W') {               // Bei Code 'W' Rücksprung zum Anfang
      if (wiederholung==false) {n=-1; wiederholung=true; } 
    } else {
      ton(speakerPin, frequenz(toneCode[n]), dauer(toneCode[n]));
      // tone(speakerPin, frequenz(toneCode[n])); delay(dauer(toneCode[n])); noTone(speakerPin);  
    }  
  }
  delay(5000);                                       // Nach 5 Sekunden nächster Durchgang
}

